import os
import sys
import time
import signal
import logging
import RPi.GPIO as GPIO
from argparse import ArgumentParser
from distutils.util import strtobool
from voucher import __version__ as version
from voucher import mqtt, mqtt_client, unifi, BUTTON_PIN_NUMBER


def signal_handler(sig, frame):
    """Catch interuption signal and gracefully shut down"""
    global mqtt_client
    if mqtt_client:
        logging.info("Shuttin down...")
        mqtt_client.disconnect()
        mqtt_client.loop_stop()
    logging.info("... program exit")
    sys.exit(0)


def create_parser(args=sys.argv[1:]):
    """Create a argparse.ArgumentParser
    Creates a ArgumentParser based on command line
    Parameters
    ----------
    args: list, optional
        a list that represents how it was called from the command line
        if not specified the list is build wid sys.argv[1:]
    Returns
    -------
    argparse.ArgumentParser
    """

    parser = ArgumentParser()
    parser.add_argument(
        "--version", "-v", action="version", version=f"%(prog)s {version}"
    )

    parser.add_argument(
        "--log-level",
        "-l",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        type=str.upper,
        help="Log level for application",
        default=os.getenv("LOG_LEVEL", "WARNING").upper(),
    )

    parser.add_argument(
        "--mqtt-host",
        help="mqtt host to connect to. Defaults to localhost.",
        default=os.getenv("MQTT_HOST", "localhost"),
    )

    parser.add_argument(
        "--mqtt-client-id",
        help="id to use for this client. Defaults to wifi-voucher_ appended with the process id",
        default=os.getenv("MQTT_CLIENT_ID", f"wifi-voucher_{os.getpid()}"),
    )
    parser.add_argument(
        "--capath",
        help="path to a directory containing trusted CA certificates to enable encrypted communication, defaults to /etc/ssl/certsc/ca-certificates.crt",
        default=os.getenv("CAPATH", "/etc/ssl/certs/ca-certificates.crt"),
    )
    parser.add_argument(
        "--mqtt-tls-version",
        choices=["tlsv1.1", "tlsv1.2", "tlsv1.3"],
        help="TLS protocol version, can be one of tlsv1.3 tlsv1.2 or tlsv1.1. Defaults to tlsv1.2 if available.",
        default=os.getenv("MQTT_TLS_VERSION", "tlsv1.2"),
    )
    parser.add_argument(
        "--mqtt-insecure",
        help="do not check that the server certificate hostname matches the remote hostname.",
        action="store_true",
    )
    parser.add_argument(
        "--mqtt-user",
        help="provide a username for mqtt broker connection",
        default=os.getenv("MQTT_USER", None),
    )
    parser.add_argument(
        "--mqtt-password",
        help="provide a password for mqtt broker connection",
        default=os.getenv("MQTT_PASSWORD", None),
    )
    parser.add_argument(
        "--mqtt-topic",
        help="mqtt topic to subscribe to. defautls to wifi-voucher",
        default=os.getenv("MQTT_TOPIC", "wifi-voucher"),
    )

    parser.add_argument(
        "--mqtt-protocol-version",
        help="specify the version of the MQTT protocol to use when connecting. Defaults to mqttv311.",
        choices=["mqttv31", "mqttv311", "mqttv5"],
        default=os.getenv("MQTT_PROTOCOL_VERSION", "mqttv311"),
    )
    parser.add_argument(
        "--mqtt-tls",
        action="store_true",
        help="Use TLS when connecting to mqtt broker",
        default=bool(strtobool(os.getenv("MQTT_TLS", "False"))),
    )
    parser.add_argument(
        "--mqtt-port",
        type=int,
        help="network port to connect to. Defaults to 1883 for plain MQTT and 8883 for MQTT over TLS",
        default=os.getenv("MQTT_PORT", None),
    )
    parser.add_argument(
        "--unifi-host",
        help="host where Unifi Controller is located. Default is localhost",
        default=os.getenv("UNIFI_HOST", "localhost"),
    )
    parser.add_argument(
        "--unifi-port",
        help="port that Unifi controller is listing on. Default 8443",
        default=os.getenv("UNIFI_PORT", "8443"),
        type=int,
    )
    parser.add_argument(
        "--unifi-insecure",
        help="don't validate tls certificates",
        action="store_true",
        default=bool(strtobool(os.getenv("UNIFI_INSECURE", "False"))),
    )
    parser.add_argument(
        "--unifi-user",
        help="User to login to Unifi controller",
        default=os.getenv("UNIFI_USER", None),
    )
    parser.add_argument(
        "--unifi-password",
        help="Password for logging in to unifi controller",
        default=os.getenv("UNIFI_PASSWORD", None),
    )
    parser.add_argument(
        "--unifi-voucher-expire-unit",
        choices=["minute", "hour", "day", "week", "year"],
        help="Set the unit that --unifi-voucher-expire-number is using, default day",
        default=os.getenv("UNIFI_VOUCHER_EXPIRE_UNIT", "day"),
    )
    parser.add_argument(
        "--unifi-voucher-expire-number",
        type=int,
        help="Set the number on how long the voucher is valid, default 1",
        default=os.getenv("UNIFI_VOUCHER_EXPIRE_NUMBER", "1"),
    )
    parser.add_argument(
        "--unifi-voucher-number",
        type=int,
        help="Number of vouchers to create. Default 1",
        default=os.getenv("UNIFI_VOUCHER_NUMBER", 1),
    )
    parser.add_argument(
        "--unifi-voucher-quota",
        type=int,
        help="Number of times the voucher can be used. Default 1",
        default=os.getenv("UNIFI_VOUCHER_QUOTA", 1),
    )
    parser.add_argument(
        "--unifi-voucher-limit-up",
        type=int,
        help="Limit the bandwidth for uploads, in Kbps. Default unlimited",
        default=os.getenv("UNIFI_VOUCHER_LIMIT_UP", None),
    )

    parser.add_argument(
        "--unifi-voucher-limit-down",
        type=int,
        help="Limit the bandwidth for downloads, in Kbps. Default unlimited",
        default=os.getenv("UNIFI_VOUCHER_LIMIT_DOWN", None),
    )
    parser.add_argument(
        "--unifi-voucher-limit-byte",
        type=int,
        help="Limit the total MB transfer for the voucher, in MB. Default unlimited",
        default=os.getenv("UNIFI_VOUCHER_LIMIT_BYTE", None),
    )
    parser.add_argument(
        "--unifi-site",
        help="Site in the Unifi controller to create vouchers for, default is 'default'",
        default=os.getenv("UNIFI_SITE", "default"),
    )
    parser.add_argument(
        "--printer", help="CUPS printer name", default=os.getenv("PRINTER", None)
    )

    return parser


def main():
    """The main program logic"""

    global mqtt_client

    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)

    parser = create_parser()
    args = parser.parse_args()

    # set up the logging
    logging.basicConfig(
        level=args.log_level, format="%(asctime)s - %(levelname)s - %(message)s"
    )
    logging.info(f"starting version {version}")
    logging.debug(args)

    if bool(args.mqtt_user) != bool(args.mqtt_password):
        logging.error(
            "Both MQTT Username and Password needs to be set, or none of them"
        )
        sys.exit(1)

    if args.unifi_user == None or args.unifi_password == None:
        logging.error("Both Unifi user and Unifi password needs to be set")
        sys.exit(1)

    if args.printer == None:
        logging.error("You need to specify a printer")
        sys.exit(2)

    mqtt_client = mqtt.create_client(args)

    if mqtt_client:
        mqtt_client.loop_start()

    GPIO.setwarnings(False)
    GPIO.setmode(GPIO.BCM)
    GPIO.setup(BUTTON_PIN_NUMBER, GPIO.IN, pull_up_down=GPIO.PUD_UP)
    GPIO.add_event_detect(BUTTON_PIN_NUMBER, GPIO.FALLING, bouncetime=300)

    while True:
        if GPIO.event_detected(BUTTON_PIN_NUMBER):
            logging.debug("Button was pressed")
            for voucher in unifi.create_voucher(args):
                unifi.print_voucher(args, voucher)
        time.sleep(0.3)
