import ssl
import sys
import json
import logging
from voucher import unifi
from argparse import Namespace
import paho.mqtt.client as mqtt


def on_connect(client, userdata, flags, rc):
    if rc == 0:
        logging.info(f"Connected to MQTT with result code {rc}")
        client.subscribe(client.args.mqtt_topic)
    else:
        logging.warning("Connection to MQTT failed, result code {rc}")
        client.reconnect()


def on_disconnect(client, userdata, rc):
    if rc != 0:
        logging.warning(
            f"Unexpected disconnected from MQTT, result code {rc}. Reconnecting..."
        )
        client.reconnect()
    else:
        logging.info("Disconnected from MQTT successfully")


def on_message(client, userdata, message):
    logging.debug(
        f"MQTT Message received ({message.topic}): {message.payload.decode('UTF-8')}"
    )
    args = vars(client.args)
    try:
        recived = json.loads(message.payload.decode("UTF-8"))
    except json.decoder.JSONDecodeError as err:
        logging.warning(f"Couldn't parse json response {err}")
        return
    try:
        args.update(recived)
    except ValueError as err:
        logging.error(f"The message could be parsed as json, but was not a dict")
        return

    vouchers = unifi.create_voucher(Namespace(**args))
    for voucher in vouchers:
        unifi.print_voucher(Namespace(**args), voucher)


def create_client(args):
    if args.mqtt_tls_version == "tlsv1.1":
        tls_version = ssl.PROTOCOL_TLSv1_1
    elif args.mqtt_tls_version == "tlsv1.2":
        tls_version = ssl.PROTOCOL_TLSv1_2
    elif args.mqtt_tls_version == "tlsv1.3":
        tls_version = ssl.PROTOCOL_TLSv1_3

    if args.mqtt_port == None:
        if args.mqtt_tls:
            mqtt_port = 8883
        else:
            mqtt_port = 1883

    client = mqtt.Client(args.mqtt_client_id)
    client.args = args
    client.enable_logger()
    if args.mqtt_user:
        client.username_pw_set(args.mqtt_user, password=args.mqtt_password)
    client.on_connect = on_connect
    client.on_message = on_message
    client.on_disconnect = on_disconnect
    if args.mqtt_tls:
        if args.mqtt_insecure:
            client.tls_set(
                ca_certs=args.capath,
                certfile=None,
                keyfile=None,
                cert_reqs=ssl.CERT_NONE,
                tls_version=tls_version,
                ciphers=None,
            )
        else:
            client.tls_set(
                ca_certs=args.capath,
                certfile=None,
                keyfile=None,
                cert_reqs=ssl.CERT_REQUIRED,
                tls_version=tls_version,
                ciphers=None,
            )

    try:
        client.connect(args.mqtt_host, port=int(mqtt_port))
    except Exception as err:
        logging.error(
            f"Couln't connecto to broker on {args.mqtt_host}:{mqtt_port}: {err}"
        )
        return None
    return client
